/**
	@section COPYRIGHT
	Copyright (C) 2019 Consequential Robotics Ltd
	
	@section AUTHOR
	Consequential Robotics http://consequentialrobotics.com
	
	@section LICENSE
	For a full copy of the license agreement, and a complete
	definition of "The Software", see LICENSE in the MDK root
	directory.
	
	Subject to the terms of this Agreement, Consequential
	Robotics grants to you a limited, non-exclusive, non-
	transferable license, without right to sub-license, to use
	"The Software" in accordance with this Agreement and any
	other written agreement with Consequential Robotics.
	Consequential Robotics does not transfer the title of "The
	Software" to you; the license granted to you is not a sale.
	This agreement is a binding legal agreement between
	Consequential Robotics and the purchasers or users of "The
	Software".
	
	THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY
	KIND, EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE
	WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR
	PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS
	OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR
	OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR
	OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE
	SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.

	This is a MIRO ROS client template for C++.
*/

#include <iostream>
using namespace std;

#include <ros/ros.h>
#include <geometry_msgs/TwistStamped.h>
#include "miro2_msg/sensors_package.h"



////////////////////////////////////////////////////////////////
//	ROS interface

struct ROSINTERF
{
	//	ROS handle
	ros::NodeHandle h_ros;

	//	publishers
	ros::Publisher pub_cmd_vel;

	//	subscribers
	ros::Subscriber sub_package;
};

static ROSINTERF* rosinterf;

void ros_init(string robot_name)
{
	//	initialization
	cout << "initializing ROS..." << endl;
	int argc = 0;
	char **argv = NULL;
	ros::init(argc, argv, "client_template_" + robot_name);

	//	create
	rosinterf = new ROSINTERF;
}

static float g_dtheta;

void callback_package(const miro2_msg::sensors_package::ConstPtr& msg)
{
	/*
		This simple controller all executes within the context of
		this callback, which itself fires when the sensors package
		message arrives from the robot.

		The controller is Braitenberg-esque: if the light level is
		higher to one side of the robot or the other, the robot will
		turn towards the light; if the light levels are about the
		same, the robot will wait.
	*/

	//	extract forward light sensor data
	float l = msg->light.data[0];
	float r = msg->light.data[1];

	//	determine wheel motion
	const float dtheta_dead_zone = 0.1f;
	float dtheta = l - r;
	if (fabs(dtheta) < dtheta_dead_zone)
	{
		dtheta = 0.0f;
	}

	//	send
	const float gain = 8.0f;
	g_dtheta = dtheta * gain;

	//	report
	cout << "update: " << l << ", " << r << ", " << (l - r) << ", " << dtheta << endl;
}



////////////////////////////////////////////////////////////////
//	MAIN

#define R_QUEUE_SIZE 1
#define S_QUEUE_SIZE 1

int main()
{
	//	report
	cout << "enter" << endl;

	//	args
	string robot_name = getenv("MIRO_ROBOT_NAME");
	string topic_base_name = "/" + robot_name + "/";

	//	init
	ros_init(robot_name);

	//	subscribe
	string topic_name = topic_base_name + "/sensors/package";
	cout << "subscribing to \"" << topic_name << "\"..." << endl;
	rosinterf->sub_package = rosinterf->h_ros.subscribe(topic_name, R_QUEUE_SIZE, callback_package);

	//	publish
	topic_name = topic_base_name + "/control/cmd_vel";
	cout << "publishing on \"" << topic_name << "\"..." << endl;
	rosinterf->pub_cmd_vel = rosinterf->h_ros.advertise<geometry_msgs::TwistStamped>(topic_name.c_str(), S_QUEUE_SIZE);

	//	outputs
	geometry_msgs::TwistStamped msg_cmd_vel;

	//	loop
	while (ros::ok())
	{
		//	spin
		ros::spinOnce();

		//	send output
		msg_cmd_vel.twist.angular.z = g_dtheta;
		rosinterf->pub_cmd_vel.publish(msg_cmd_vel);

		//	control at 10Hz
		usleep(100000);
	}

	//	report
	cout << "exit" << endl;
	return 0;
}



